@@ -1,9 +1,8 @@
 import logging
 import time
-import sys
 import os
-import json
 import re
+import json
 import six
 import yaml
 import tempfile
@@ -14,21 +13,19 @@ from gql.transport.requests import RequestsHTTPTransport
 from six.moves import urllib
 from functools import partial
 import shutil
-import hashlib
-import base64
 import requests
 import platform
 
 import wandb
-from wandb.core import termlog
-from wandb import Error, __version__
-from wandb import artifacts
+from wandb.errors.term import termlog
+from wandb import __version__
 from wandb import util
-from wandb.retry import retriable
-from wandb.summary import HTTPSummary
+from wandb.old.retry import retriable
+from wandb.old.summary import HTTPSummary
 from wandb import env
+from wandb.apis.normalize import normalize_exceptions
 from wandb.apis.internal import Api as InternalApi
-from wandb.apis import normalize_exceptions
+from wandb.interface import artifacts
 
 logger = logging.getLogger(__name__)
 
@@ -115,10 +112,15 @@ fragment ArtifactFragment on Artifact {
     updatedAt
     labels
     metadata
+    versionIndex
     aliases {
         artifactCollectionName
         alias
     }
+    artifactSequence {
+        id
+        name
+    }
     artifactType {
         id
         name
@@ -127,7 +129,7 @@ fragment ArtifactFragment on Artifact {
         id
         file {
             id
-            url
+            directUrl
         }
     }
 }
@@ -268,23 +270,15 @@ class Api(object):
         self._runs = {}
 
     def _parse_project_path(self, path):
-        """Parses paths in the following formats:
-
-        entity/project
-        project
-
-        entity is optional and will fall back to the current logged in user
-        """
-        parts = path.split("/")
-        entity = self.settings['entity'] or self.default_entity
+        """Returns project and entity for project specified by path"""
         project = self.settings['project']
+        entity = self.settings['entity'] or self.default_entity
+        if path is None:
+            return entity, project
+        parts = path.split('/', 1)
         if len(parts) == 1:
-            project = parts[1]
-        elif len(parts) == 2:
-            entity, project = parts[0], parts[1]
-        else:
-            raise ValueError('Invalid project path: %s' % path)
-        return entity, project
+            return entity, path
+        return parts
 
     def _parse_path(self, path):
         """Parses paths in the following formats:
@@ -293,8 +287,9 @@ class Api(object):
         path: entity/project/run_id
         docker: entity/project:run_id
 
-        entity is optional and will fall back to the current logged in user.
+        entity is optional and will fallback to the current logged in user.
         """
+        run = self.settings['run']
         project = self.settings['project']
         entity = self.settings['entity'] or self.default_entity
         parts = path.replace("/runs/", "/").strip("/ ").split("/")
@@ -330,17 +325,6 @@ class Api(object):
             return entity, parts[0], parts[1]
         return parts
 
-    def _parse_project_path(self, path):
-        """Returns project and entity for project specified by path"""
-        project = self.settings['project']
-        entity = self.settings['entity'] or self.default_entity
-        if path is None:
-            return entity, project
-        parts = path.split('/', 1)
-        if len(parts) == 1:
-            return entity, path
-        return parts
-
     def projects(self, entity=None, per_page=200):
         """Get projects for a given entity.
         Args:
@@ -500,7 +484,6 @@ class Api(object):
         """
         if name is None:
             raise ValueError('You must specify name= to fetch an artifact.')
-
         entity, project, artifact_name = self._parse_artifact_path(name)
         artifact = Artifact(self.client, entity, project, artifact_name)
         if type is not None and artifact.type != type:
@@ -744,6 +727,8 @@ class Runs(Paginator):
 
     def convert_objects(self):
         objs = []
+        if self.last_response is None or self.last_response.get('project') is None:
+            raise ValueError("Could not find project %s" % self.project)
         for run_response in self.last_response['project']['runs']['edges']:
             run = Run(self.client, self.entity, self.project, run_response["node"]["name"], run_response["node"])
             objs.append(run)
@@ -756,6 +741,8 @@ class Runs(Paginator):
                                       run.sweep_name, withRuns=False)
                     self._sweeps[run.sweep_name] = sweep
 
+                if sweep is None:
+                    continue
                 run.sweep = sweep
                 if run.id not in sweep.runs_by_id:
                     sweep.runs_by_id[run.id] = run
@@ -978,7 +965,8 @@ class Run(Attrs):
         ''')
 
         response = self._exec(query, specs=[json.dumps(spec)])
-        return [line for line in response['project']['run']['sampledHistory']]
+        # sampledHistory returns one list per spec, we only send one spec
+        return response['project']['run']['sampledHistory'][0]
 
     def _full_history(self, samples=500, stream="default"):
         node = "history" if stream == "default" else "events"
@@ -1092,6 +1080,55 @@ class Run(Attrs):
     def used_artifacts(self, per_page=100):
         return RunArtifacts(self.client, self, mode="used", per_page=per_page)
 
+    @normalize_exceptions
+    def use_artifact(self, artifact):
+        """ Declare an artifact as an input to a run.
+
+        Args:
+            artifact (:obj:`Artifact`): An artifact returned from
+                `wandb.Api().artifact(name)`
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        api = InternalApi(default_settings={
+            "entity": self.entity, "project": self.project})
+        api.set_current_run_id(self.id)
+
+        if isinstance(artifact, Artifact):
+            api.use_artifact(artifact.id)
+            return artifact
+        elif isinstance(artifact, wandb.Artifact):
+            raise ValueError("Only existing artifacts are accepted by this api. "
+                             "Manually create one with `wandb artifacts put`")
+        else:
+            raise ValueError('You must pass a wandb.Api().artifact() to use_artifact')
+
+    @normalize_exceptions
+    def log_artifact(self, artifact, aliases=None):
+        """ Declare an artifact as output of a run.
+
+        Args:
+            artifact (:obj:`Artifact`): An artifact returned from
+                `wandb.Api().artifact(name)`
+            aliases (list, optional): Aliases to apply to this artifact
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        api = InternalApi(default_settings={
+            "entity": self.entity, "project": self.project})
+        api.set_current_run_id(self.id)
+
+        if isinstance(artifact, Artifact):
+            artifact_collection_name = artifact.name.split(':')[0]
+            api.create_artifact(artifact.type, artifact_collection_name,
+                                artifact.digest, aliases=aliases)
+            return artifact
+        elif isinstance(artifact, wandb.Artifact):
+            raise ValueError("Only existing artifacts are accepted by this api. "
+                             "Manually create one with `wandb artifacts put`")
+        else:
+            raise ValueError('You must pass a wandb.Api().artifact() to use_artifact')
+
     @property
     def summary(self):
         if self._summary is None:
@@ -1795,6 +1832,8 @@ class ProjectArtifactTypes(Paginator):
         self.variables.update({'cursor': self.cursor})
 
     def convert_objects(self):
+        if self.last_response['project'] is None:
+            return []
         return [ArtifactType(self.client, self.entity, self.project, r["node"]["name"], r["node"])
                 for r in self.last_response['project']['artifactTypes']['edges']]
 
@@ -2059,11 +2098,19 @@ class Artifact(object):
         self.entity = entity
         self.project = project
         self.artifact_name = name
-        self._collection_name = None
         self._attrs = attrs
-        self._metadata = {}
         if self._attrs is None:
             self._load()
+        self._metadata = self._attrs.get("metadata", None)
+        self._description = self._attrs.get("description", None)
+        self._sequence_name = self._attrs["artifactSequence"]["name"]
+        self._version_index = self._attrs.get("versionIndex", None)
+        self._aliases = [
+            a["alias"]
+            for a in self._attrs["aliases"]
+            if not re.match(r"^v\d+$", a["alias"])
+            and a["artifactCollectionName"] == self._sequence_name
+        ]
         self._manifest = None
         self._is_downloaded = False
 
@@ -2075,12 +2122,13 @@ class Artifact(object):
     def metadata(self):
         return self._metadata
 
-    # @property
-    # def path(self):
-    #     # TODO: This is a different style than the rest of the paths. The rest of the
-    #     # paths don't include the object type (which makes them hard to distinguish).
-    #     # We should maybe use URIs here.
-    #     return '%s/%s/artifact/%s/%s' % (self.entity, self.project, self.artifact_type_name, self.artifact_name)
+    @metadata.setter
+    def metadata(self, metadata):
+        self._metadata = metadata
+
+    @property
+    def manifest(self):
+        return self._load_manifest()
 
     @property
     def digest(self):
@@ -2104,11 +2152,11 @@ class Artifact(object):
 
     @property
     def description(self):
-        return self._attrs["description"]
+        return self._description
 
     @description.setter
     def description(self, desc):
-        self._attrs["description"] = desc
+        self._description = desc
 
     @property
     def type(self):
@@ -2116,19 +2164,38 @@ class Artifact(object):
 
     @property
     def name(self):
-        """Stable name you can use to fetch this artifact."""
-        # TODO: All this logic should move to the backend.
-        if ":" not in self.artifact_name:
-            # this is a digest lookup
-            return self.artifact_name
-        artifact_collection_name = self.artifact_name.split(':')[0]
-        for alias in self._attrs["aliases"]:
-            if alias["artifactCollectionName"] == artifact_collection_name and re.match(r"^v\d+$", alias["alias"]):
-                return '%s:%s' % (artifact_collection_name, alias["alias"])
-        
-        raise ValueError('Unexpected API result.')
+        if self._version_index is None:
+            return self.digest
+        return '%s:v%s' % (self._sequence_name, self._version_index)
 
-    def new_file(self, name):
+    @property
+    def aliases(self):
+        return self._aliases
+
+    @aliases.setter
+    def aliases(self, aliases):
+        for alias in aliases:
+            if any(char in alias for char in ['/', ':']):
+                raise ValueError('Invalid alias "%s", slashes and colons are disallowed' % alias)
+        self._aliases = aliases
+
+    def delete(self):
+        """Delete artifact and it's files."""
+        mutation = gql('''
+        mutation deleteArtifact($id: ID!) {
+            deleteArtifact(input: {artifactID: $id}) {
+                artifact {
+                    id
+                }
+            }
+        }
+        ''')
+        self.client.execute(mutation, variable_values={
+            "id": self.id,
+        })
+        return True
+
+    def new_file(self, name, mode=None):
         raise ValueError('Cannot add files to an artifact once it has been saved')
 
     def add_file(self, path, name=None):
@@ -2150,11 +2217,30 @@ class Artifact(object):
 
         class ArtifactEntry(object):
             @staticmethod
-            def download():
+            def copy(cache_path, target_path):
+                # can't have colons in Windows
+                if platform.system() == "Windows":
+                    head, tail = os.path.splitdrive(target_path)
+                    target_path = head + tail.replace(":", "-")
+
+                need_copy = (not os.path.isfile(target_path)
+                             or os.stat(cache_path).st_mtime != os.stat(target_path).st_mtime)
+                if need_copy:
+                    util.mkdir_exists_ok(os.path.dirname(target_path))
+                    # We use copy2, which preserves file metadata including modified
+                    # time (which we use above to check whether we should do the copy).
+                    shutil.copy2(cache_path, target_path)
+                return target_path
+
+            @staticmethod
+            def download(root=None):
                 if entry.ref is not None:
                     return storage_policy.load_reference(self, name, manifest.entries[name], local=True)
 
-                return storage_policy.load_file(self, name, manifest.entries[name])
+                cache_path = storage_policy.load_file(self, name, manifest.entries[name])
+                if root is not None:
+                    return ArtifactEntry().copy(cache_path, os.path.join(root, name))
+                return cache_path
 
             @staticmethod
             def ref():
@@ -2185,7 +2271,7 @@ class Artifact(object):
         nfiles = len(manifest.entries)
         size = sum(e.size for e in manifest.entries.values())
         log = False
-        if nfiles  > 5000 or size > 50 * 1024 * 1024:
+        if nfiles > 5000 or size > 50 * 1024 * 1024:
             log = True
         if log:
             termlog('Downloading large artifact %s, %.2fMB. %s files... ' % (
@@ -2228,23 +2314,8 @@ class Artifact(object):
         return self._download_file(list(manifest.entries)[0], root)
 
     def _download_file(self, name, dirpath):
-        # download file into cache
-        cache_path = self.get_path(name).download()
-        # copy file into target dir
-        target_path = os.path.join(dirpath, name)
-        # can't have colons in Windows
-        if platform.system() == "Windows":
-            head, tail = os.path.splitdrive(target_path)
-            target_path = head + tail.replace(":", "-")
-
-        need_copy = (not os.path.isfile(target_path)
-            or os.stat(cache_path).st_mtime != os.stat(target_path).st_mtime)
-        if need_copy:
-            util.mkdir_exists_ok(os.path.dirname(target_path))
-            # We use copy2, which preserves file metadata including modified
-            # time (which we use above to check whether we should do the copy).
-            shutil.copy2(cache_path, target_path)
-        return target_path
+        # download file into cache and copy to target dir
+        return self.get_path(name).download(dirpath)
 
     @normalize_exceptions
     def save(self):
@@ -2252,33 +2323,36 @@ class Artifact(object):
         Persists artifact changes to the wandb backend.
         """
         mutation = gql('''
-        mutation updateArtifact($entity: String!, $type: String!, $project: String!, $digest: String!,
-             $description: String, $metadata: JSONString, $aliases: [ArtifactAliasInput!]) {
-            createArtifact(input: {
-                entityName: $entity, projectName: $project, digest: $digest, artifactTypeName: $type,
-                description: $description, metadata: $metadata, aliases: $aliases}) {
+        mutation updateArtifact(
+            $artifactID: ID!,
+            $description: String,
+            $metadata: JSONString,
+            $aliases: [ArtifactAliasInput!]
+        ) {
+            updateArtifact(input: {
+                artifactID: $artifactID,
+                description: $description,
+                metadata: $metadata,
+                aliases: $aliases
+            }) {
                 artifact {
                     id
                 }
             }
         }
         ''')
-        res = self.client.execute(mutation, variable_values={
-            "entity": self.entity,
-            "project": self.project,
-            "digest": self.digest,
+        self.client.execute(mutation, variable_values={
+            "artifactID": self.id,
             "description": self.description,
             "metadata": util.json_dumps_safer(self.metadata),
-            "aliases": self._aliases()
+            "aliases": [{
+                "artifactCollectionName": self._sequence_name,
+                "alias": alias,
+
+            } for alias in self._aliases]
         })
         return True
 
-    def _aliases(self):
-        if ":" in self.artifact_name:
-            collection, alias = self.artifact_name.split(":")
-            return [{"artifactCollectionName": collection, "alias": alias}]
-        return []
-
     def verify(self, root=None):
         """Verify an artifact by checksumming its downloaded contents.
 
@@ -2341,10 +2415,8 @@ class Artifact(object):
             raise ValueError('Project %s/%s does not contain artifact: "%s"' % (
                 self.entity, self.project, self.artifact_name))
         self._attrs = response['project']['artifact']
-        if 'metadata' in response['project']['artifact']:
-            metadata = response['project']['artifact']['metadata']
-            if metadata is not None:
-                self._metadata = json.loads(metadata)
+        if 'metadata' in response['project']['artifact'] and response['project']['artifact']['metadata']:
+            self._metadata = json.loads(response['project']['artifact']['metadata'])
         return self._attrs
 
     # The only file should be wandb_manifest.json
@@ -2353,12 +2425,10 @@ class Artifact(object):
 
     def _load_manifest(self):
         if self._manifest is None:
-            index_file_url = self._attrs['currentManifest']['file']['url']
-            with requests.get(index_file_url, auth=("api", Api().api_key)) as req:
-                json_resp = json.loads(req.content)
-                if "error" in json_resp:
-                    raise ValueError("Failed to download manifest file: {}".format(json_resp["error"]))
-                self._manifest = artifacts.ArtifactManifest.from_manifest_json(self, json_resp)
+            index_file_url = self._attrs['currentManifest']['file']['directUrl']
+            with requests.get(index_file_url) as req:
+                req.raise_for_status()
+                self._manifest = artifacts.ArtifactManifest.from_manifest_json(self, json.loads(req.content))
         return self._manifest
 
 class ArtifactVersions(Paginator):
